import datetime
import os
import re
import socket
import time

from generic_module import Module


class EfoyModule(Module):
    def __init__(self, args):
        expected_params = {"host": "192.168.4.10", "port": 10001, "polling_interval": 3600, "recv_buffer_size": 2048, "save_to_disk": 1}
        Module.__init__(self, args, expected_params)

    def generate_path(self, mod):
        pth = self.sites_path
        pth = pth + "/" + self.sname + "/ext"
        if not os.path.exists(pth):
            os.mkdir(pth)
        pth = pth + "/" + mod
        if not os.path.exists(pth):
            os.mkdir(pth)
        pth = pth + "/efoy"
        if not os.path.exists(pth):
            os.mkdir(pth)
        pth = pth + "/" + self.name
        if not os.path.exists(pth):
            os.mkdir(pth)
        return pth

    def connect_tcp(self):
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            self.s.connect((self.host, self.port))
            self.s.settimeout(2)
            self.logger.info('Connected to {}:{}'.format(self.host, self.port))
            return 0
        except socket.error as e:
            self.logger.warning('Connection to {0}:{1} refused ({2})'.format(self.host, self.port, e))
            return 1
        except:
            self.logger.exception('Failed to connect:')
            return 1

    def run(self):
        self.alive = True
        self.end = False
        if self.connect_tcp() == 1:
            self.logger.info('Connection failed.')
            self.alive = False
            return
        try:
            line = ""
            bogus_count = 0  # for exiting thread
            sent = False
            sent_at = time.time()
            while True:
                # only do this once in a while
                if not sent or time.time() - sent_at > self.polling_interval:
                    self.logger.debug('Sending cartridge request ...')
                    self.s.send(b'cartridge\r')
                    sent = True
                    sent_at = time.time()
                if self.end:
                    self.alive = False
                    return
                received = ""
                try:
                    received = self.s.recv(self.recv_buffer_size).decode(errors='replace')
                    self.logger.debug('Received response: {}'.format(received))
                    if received == "":
                        self.logger.warning('Received an empty string, exiting thread.')
                        self.alive = False
                        return
                except socket.timeout:
                    self.logger.warning('Connection timed out.')
                except socket.error:
                    self.logger.debug('{0} with buffer [{1}] closed.'.format(self.s, self.recv_buffer_size))
                    self.logger.warning('Connection closed.')
                    self.alive = False
                    return
                line = line + received
                a = re.search('M28 \(([0-9]+)%\)', line)
                val = 0
                if a:
                    try:
                        val = a.group(1)
                        line = ""
                    except:
                        self.logger.exception('An error while parsing received bytes:')
                        bogus_count += 1
                else:
                    if bogus_count == 15:
                        self.logger.info("Exiting thread " + self.name + " due to recieved data's non well-formity, end result being: " + line)
                        self.alive = False
                        return
                    bogus_count += 1
                    continue
                for key, mod in self.downstream_modules_dict.items():
                    mod.set_upstream_info(self.get_name(), "fuel_level", val)
                if self.save_to_disk != 0:
                    pth = self.generate_path("fuel")
                    tm = datetime.datetime.now().strftime('%Y-%m-%d-%H-%M-%S')
                    date = datetime.datetime.now().strftime('%Y-%m-%d')
                    header = "ts\tfuel_level[%]"
                    fd = open(pth + "/" + date + ".csv", "a+")
                    if not os.path.exists(pth + "/" + date + ".csv") or os.path.getsize(pth + "/" + date + ".csv") == 0:
                        fd.write(header + "\n")
                    fd.write(tm + "\t" + str(val) + "\n")
                    fd.close()
        except:
            self.logger.exception('General error:')
            self.alive = False
            return
